@@ -10,6 +10,8 @@
 #include <librsvg/rsvg.h>
 #endif
 
+#include <glib/gi18n.h>
+
 #include "eog-config-keys.h"
 #include "eog-enum-types.h"
 #include "eog-scroll-view.h"
@@ -26,10 +28,6 @@
 /* Scroll step increment */
 #define SCROLL_STEP_SIZE 32
 
-/* Maximum zoom factor */
-#define MAX_ZOOM_FACTOR 20
-#define MIN_ZOOM_FACTOR 0.02
-
 #define CHECK_MEDIUM 8
 #define CHECK_BLACK "#000000"
 #define CHECK_DARK "#555555"
@@ -37,11 +35,8 @@
 #define CHECK_LIGHT "#cccccc"
 #define CHECK_WHITE "#ffffff"
 
-/* Default increment for zooming.  The current zoom factor is multiplied or
- * divided by this amount on every zooming step.  For consistency, you should
- * use the same value elsewhere in the program.
- */
-#define IMAGE_VIEW_ZOOM_MULTIPLIER 1.05
+/* Time used for the realing animation of the overlaid buttons */
+#define OVERLAY_REVEAL_ANIM_TIME (500U) /* ms */
 
 #if 0
 /* Progressive loading state */
@@ -184,6 +179,12 @@ struct _EogScrollViewPrivate {
 	gdouble initial_zoom;
 	EogRotationState rotate_state;
 	EogPanAction pan_action;
+
+	GtkWidget *overlay;
+	GtkWidget *left_revealer;
+	GtkWidget *right_revealer;
+	GtkWidget *bottom_revealer;
+	GSource   *overlay_timeout_source;
 };
 
 static void scroll_by (EogScrollView *view, int xofs, int yofs);
@@ -414,7 +415,7 @@ eog_scroll_view_set_cursor (EogScrollView *view, EogScrollViewCursor new_cursor)
 			gdk_window_set_cursor (gtk_widget_get_window (widget), NULL);
 			break;
                 case EOG_SCROLL_VIEW_CURSOR_HIDDEN:
-                        cursor = gdk_cursor_new (GDK_BLANK_CURSOR);
+			cursor = gdk_cursor_new_for_display (display, GDK_BLANK_CURSOR);
                         break;
 		case EOG_SCROLL_VIEW_CURSOR_DRAG:
 			cursor = gdk_cursor_new_for_display (display, GDK_FLEUR);
@@ -1313,6 +1314,8 @@ set_zoom (EogScrollView *view, double zoom,
 	int xofs, yofs;
 	double x_rel, y_rel;
 
+	g_assert (zoom > 0.0);
+
 	priv = view->priv;
 
 	if (priv->pixbuf == NULL)
@@ -2670,6 +2673,80 @@ sv_rgba_to_string_mapping (const GValue       *value,
 	return variant;
 }
 
+static void
+_clear_overlay_timeout (EogScrollView *view)
+{
+	EogScrollViewPrivate *priv = view->priv;
+
+	if (priv->overlay_timeout_source != NULL) {
+		g_source_unref (priv->overlay_timeout_source);
+		g_source_destroy (priv->overlay_timeout_source);
+	}
+
+	priv->overlay_timeout_source = NULL;
+}
+static gboolean
+_overlay_timeout_cb (gpointer data)
+{
+	EogScrollView *view = EOG_SCROLL_VIEW (data);
+	EogScrollViewPrivate *priv = view->priv;
+
+	gtk_revealer_set_reveal_child (GTK_REVEALER (priv->left_revealer), FALSE);
+	gtk_revealer_set_reveal_child (GTK_REVEALER (priv->right_revealer), FALSE);
+	gtk_revealer_set_reveal_child (GTK_REVEALER (priv->bottom_revealer), FALSE);
+
+	_clear_overlay_timeout (view);
+
+	return FALSE;
+}
+static void
+_set_overlay_timeout (EogScrollView *view)
+{
+	GSource *source;
+
+	_clear_overlay_timeout (view);
+
+	source = g_timeout_source_new_seconds (2);
+	g_source_set_callback (source, _overlay_timeout_cb, view, NULL);
+
+	g_source_attach (source, NULL);
+
+	view->priv->overlay_timeout_source = source;
+}
+
+static gboolean
+_enter_overlay_event_cb (GtkWidget *widget,
+			 GdkEvent *event,
+			 gpointer user_data)
+{
+	EogScrollView *view = EOG_SCROLL_VIEW (user_data);
+
+	_clear_overlay_timeout (view);
+
+	return FALSE;
+}
+
+static gboolean
+_motion_notify_cb (GtkWidget *widget, GdkEventMotion *event, gpointer user_data)
+{
+	EogScrollView *view = EOG_SCROLL_VIEW (user_data);
+	EogScrollViewPrivate *priv = view->priv;
+	gboolean reveal_child;
+
+	reveal_child = gtk_revealer_get_reveal_child (GTK_REVEALER (priv->left_revealer));
+
+	if (!reveal_child) {
+		gtk_revealer_set_reveal_child (GTK_REVEALER (priv->left_revealer), TRUE);
+		gtk_revealer_set_reveal_child (GTK_REVEALER (priv->right_revealer), TRUE);
+		gtk_revealer_set_reveal_child (GTK_REVEALER (priv->bottom_revealer), TRUE);
+	}
+
+	/* reset timeout */
+	_set_overlay_timeout(view);
+
+	return FALSE;
+}
+
 static void
 eog_scroll_view_init (EogScrollView *view)
 {
@@ -2711,6 +2788,10 @@ eog_scroll_view_init (EogScrollView *view)
 			  view);
 
 	priv->vbar = gtk_scrollbar_new (GTK_ORIENTATION_VERTICAL, priv->vadj);
+
+	priv->overlay = gtk_overlay_new ();
+	gtk_grid_attach (GTK_GRID (view), priv->overlay, 0, 0, 1, 1);
+
 	priv->display = g_object_new (GTK_TYPE_DRAWING_AREA,
 				      "can-focus", TRUE,
 				      NULL);
@@ -2757,8 +2838,8 @@ eog_scroll_view_init (EogScrollView *view)
 	g_signal_connect (G_OBJECT (priv->display), "drag-begin",
 			  G_CALLBACK (view_on_drag_begin_cb), view);
 
-	gtk_grid_attach (GTK_GRID (view), priv->display,
-			 0, 0, 1, 1);
+	gtk_container_add (GTK_CONTAINER (priv->overlay), priv->display);
+
 	gtk_widget_set_hexpand (priv->display, TRUE);
 	gtk_widget_set_vexpand (priv->display, TRUE);
 	gtk_grid_attach (GTK_GRID (view), priv->hbar,
@@ -2820,6 +2901,108 @@ eog_scroll_view_init (EogScrollView *view)
 					   TRUE);
 	gtk_event_controller_set_propagation_phase (GTK_EVENT_CONTROLLER (priv->pan_gesture),
 						    GTK_PHASE_CAPTURE);
+
+	/* left revealer */
+	priv->left_revealer = gtk_revealer_new ();
+	gtk_revealer_set_transition_type (GTK_REVEALER (priv->left_revealer),
+					  GTK_REVEALER_TRANSITION_TYPE_CROSSFADE);
+	gtk_revealer_set_transition_duration (GTK_REVEALER (priv->left_revealer),
+					      OVERLAY_REVEAL_ANIM_TIME);
+	gtk_widget_set_halign (priv->left_revealer, GTK_ALIGN_START);
+	gtk_widget_set_valign (priv->left_revealer, GTK_ALIGN_CENTER);
+	gtk_widget_set_margin_start(priv->left_revealer, 12);
+	gtk_overlay_add_overlay (GTK_OVERLAY (priv->overlay),
+				 priv->left_revealer);
+
+	/* right revealer */
+	priv->right_revealer = gtk_revealer_new ();
+	gtk_revealer_set_transition_type (GTK_REVEALER (priv->right_revealer),
+					  GTK_REVEALER_TRANSITION_TYPE_CROSSFADE);
+	gtk_revealer_set_transition_duration (GTK_REVEALER (priv->right_revealer),
+					      OVERLAY_REVEAL_ANIM_TIME);
+	gtk_widget_set_halign (priv->right_revealer, GTK_ALIGN_END);
+	gtk_widget_set_valign (priv->right_revealer, GTK_ALIGN_CENTER);
+	gtk_widget_set_margin_end (priv->right_revealer, 12);
+	gtk_overlay_add_overlay(GTK_OVERLAY (priv->overlay),
+				priv->right_revealer);
+
+	/* bottom revealer */
+	priv->bottom_revealer = gtk_revealer_new ();
+	gtk_revealer_set_transition_type (GTK_REVEALER (priv->bottom_revealer),
+					  GTK_REVEALER_TRANSITION_TYPE_CROSSFADE);
+	gtk_revealer_set_transition_duration (GTK_REVEALER (priv->bottom_revealer),
+					      OVERLAY_REVEAL_ANIM_TIME);
+	gtk_widget_set_halign (priv->bottom_revealer, GTK_ALIGN_CENTER);
+	gtk_widget_set_valign (priv->bottom_revealer, GTK_ALIGN_END);
+	gtk_widget_set_margin_bottom (priv->bottom_revealer, 12);
+	gtk_overlay_add_overlay (GTK_OVERLAY (priv->overlay),
+				 priv->bottom_revealer);
+
+	/* overlaid buttons */
+	GtkWidget *button = gtk_button_new_from_icon_name ("go-next-symbolic",
+							   GTK_ICON_SIZE_BUTTON);
+
+	gtk_container_add(GTK_CONTAINER (priv->right_revealer), button);
+	gtk_actionable_set_action_name(GTK_ACTIONABLE (button), "win.go-next");
+	gtk_widget_set_tooltip_text (button,
+				     _("Go to the next image of the gallery"));
+	gtk_style_context_add_class (gtk_widget_get_style_context (button),
+				     GTK_STYLE_CLASS_OSD);
+
+
+	button = gtk_button_new_from_icon_name("go-previous-symbolic",
+					       GTK_ICON_SIZE_BUTTON);
+
+	gtk_container_add(GTK_CONTAINER (priv->left_revealer), button);
+	gtk_actionable_set_action_name (GTK_ACTIONABLE(button),
+					"win.go-previous");
+	gtk_widget_set_tooltip_text (button,
+				     _("Go to the previous image of the gallery"));
+	gtk_style_context_add_class (gtk_widget_get_style_context (button),
+				     GTK_STYLE_CLASS_OSD);
+
+
+	/* group rotate buttons into a box */
+	GtkWidget* bottomBox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+	gtk_style_context_add_class (gtk_widget_get_style_context (bottomBox),
+				     GTK_STYLE_CLASS_LINKED);
+
+	button = gtk_button_new_from_icon_name ("object-rotate-left-symbolic",
+						GTK_ICON_SIZE_BUTTON);
+	gtk_actionable_set_action_name (GTK_ACTIONABLE (button),
+					"win.rotate-270");
+	gtk_widget_set_tooltip_text (button,
+				     _("Rotate the image 90 degrees to the left"));
+	gtk_style_context_add_class (gtk_widget_get_style_context (button),
+				     GTK_STYLE_CLASS_OSD);
+
+	gtk_container_add (GTK_CONTAINER (bottomBox), button);
+
+	button = gtk_button_new_from_icon_name ("object-rotate-right-symbolic",
+						GTK_ICON_SIZE_BUTTON);
+	gtk_actionable_set_action_name (GTK_ACTIONABLE (button),
+					"win.rotate-90");
+	gtk_widget_set_tooltip_text (button,
+				     _("Rotate the image 90 degrees to the right"));
+	gtk_style_context_add_class (gtk_widget_get_style_context (button),
+				     GTK_STYLE_CLASS_OSD);
+	gtk_container_add (GTK_CONTAINER (bottomBox), button);
+
+	gtk_container_add (GTK_CONTAINER (priv->bottom_revealer), bottomBox);
+
+	/* Display overlay buttons on mouse movement */
+	g_signal_connect (priv->display,
+			  "motion-notify-event",
+			  G_CALLBACK (_motion_notify_cb),
+			  view);
+
+	/* Don't hide overlay buttons when above */
+	gtk_widget_add_events (GTK_WIDGET (priv->overlay),
+			       GDK_ENTER_NOTIFY_MASK);
+	g_signal_connect (priv->overlay,
+			  "enter-notify-event",
+			  G_CALLBACK (_enter_overlay_event_cb),
+			  view);
 }
 
 static void
